package java.text;

import java.util.HashMap;
import java.io.InvalidObjectException;
import java.io.Serializable;
import java.util.Map;
import java.util.Set;

public interface AttributedCharacterIterator extends CharacterIterator
{
    public abstract int getRunStart();
    
    public abstract int getRunStart(AttributedCharacterIterator.Attribute p0);
    
    public abstract int getRunStart(Set<? extends AttributedCharacterIterator.Attribute> p0);
    
    public abstract int getRunLimit();
    
    public abstract int getRunLimit(AttributedCharacterIterator.Attribute p0);
    
    public abstract int getRunLimit(Set<? extends AttributedCharacterIterator.Attribute> p0);
    
    public abstract Map<AttributedCharacterIterator.Attribute, Object> getAttributes();
    
    public abstract Object getAttribute(AttributedCharacterIterator.Attribute p0);
    
    public abstract Set<AttributedCharacterIterator.Attribute> getAllAttributeKeys();
    
    public static class Attribute implements Serializable
    {
        private String name;
        private static final Map<String, AttributedCharacterIterator.Attribute> instanceMap;
        public static final AttributedCharacterIterator.Attribute LANGUAGE;
        public static final AttributedCharacterIterator.Attribute READING;
        public static final AttributedCharacterIterator.Attribute INPUT_METHOD_SEGMENT;
        private static final long serialVersionUID = -9142742483513960612L;
        
        protected Attribute(String name) {
            this.name = name;
            if (this.getClass() == AttributedCharacterIterator.Attribute.class) {
                AttributedCharacterIterator.Attribute.instanceMap.put(name, this);
            }
        }
        
        @Override
        public final boolean equals(Object obj) {
            return super.equals(obj);
        }
        
        @Override
        public final int hashCode() {
            return super.hashCode();
        }
        
        @Override
        public String toString() {
            return this.getClass().getName() + "(" + this.name + ")";
        }
        
        protected String getName() {
            return this.name;
        }
        
        protected Object readResolve() throws InvalidObjectException {
            if (this.getClass() != AttributedCharacterIterator.Attribute.class) {
                throw new InvalidObjectException("subclass didn't correctly implement readResolve");
            }
            AttributedCharacterIterator.Attribute attribute = (AttributedCharacterIterator.Attribute)AttributedCharacterIterator.Attribute.instanceMap.get(this.getName());
            if (attribute != null) {
                return attribute;
            }
            throw new InvalidObjectException("unknown attribute name");
        }
        
        static {
            instanceMap = new HashMap(7);
            LANGUAGE = new AttributedCharacterIterator.Attribute("language");
            READING = new AttributedCharacterIterator.Attribute("reading");
            INPUT_METHOD_SEGMENT = new AttributedCharacterIterator.Attribute("input_method_segment");
        }
    }
}
